import { baseUrl } from './env';

/**
 * Sets custom text to display on the clock
 */
export const setCustomText = (newText: string) => {
	return fetch(`${baseUrl}/api/show/text/${newText}`).catch(() => {});
};

/**
 * Updates the LED colors
 */
export const setLEDcolor = (ledStatus: { hex: string }[]) => {
	return fetch(`${baseUrl}/api/lights/set`, {
		headers: {
			'Content-Type': 'application/json'
		},
		method: 'PATCH',
		body: JSON.stringify(ledStatus)
	}).catch(() => {});
};

/**
 * Turns off all LEDs
 */
export const turnOffLeds = () => {
	return fetch(`${baseUrl}/api/lights/off`).catch(() => {});
};

/**
 * Restarts the clock
 */
export const restartClock = () => {
	return fetch(`${baseUrl}/api/restart`).catch(() => {});
};

/**
 * Forces a full refresh of the clock
 */
export const forceFullRefresh = () => {
	return fetch(`${baseUrl}/api/full_refresh`).catch(() => {});
};

/**
 * Generates a random color hex code
 */
export const generateRandomColor = () => {
	return `#${Math.floor(Math.random() * 16777215)
		.toString(16)
		.padStart(6, '0')}`;
};

/**
 * Sets the active screen
 */
export const setActiveScreen = async (screenId: string) => {
	return fetch(`${baseUrl}/api/show/screen/${screenId}`);
};

/**
 * Sets the active currency
 */
export const setActiveCurrency = async (currency: string) => {
	return fetch(`${baseUrl}/api/show/currency/${currency}`);
};

/**
 * Turns on the frontlight
 */
export const turnOnFrontlight = () => {
	return fetch(`${baseUrl}/api/frontlight/on`).catch(() => {});
};

/**
 * Flashes the frontlight
 */
export const flashFrontlight = () => {
	return fetch(`${baseUrl}/api/frontlight/flash`).catch(() => {});
};

/**
 * Turns off the frontlight
 */
export const turnOffFrontlight = () => {
	return fetch(`${baseUrl}/api/frontlight/off`).catch(() => {});
};

/**
 * Toggles the timer
 */
export const toggleTimer = (currentStatus: boolean) => (e: Event) => {
	e.preventDefault();
	if (currentStatus) {
		fetch(`${baseUrl}/api/action/pause`);
	} else {
		fetch(`${baseUrl}/api/action/timer_restart`);
	}
};

/**
 * Toggles the do not disturb mode
 */
export const toggleDoNotDisturb = (currentStatus: boolean) => (e: Event) => {
	e.preventDefault();
	console.log(currentStatus);
	if (!currentStatus) {
		fetch(`${baseUrl}/api/dnd/enable`);
	} else {
		fetch(`${baseUrl}/api/dnd/disable`);
	}
};

/**
 * Uploads a file to a specified endpoint with progress, success and error callbacks
 * @param file The file to upload
 * @param endpoint The endpoint to upload to
 * @param callbacks Optional callbacks for various events during upload
 * @returns A promise that resolves when the upload is complete or fails
 */
export const uploadFile = async (
	file: File | null,
	endpoint: string,
	callbacks?: {
		onProgress?: (progress: number) => void;
		onSuccess?: () => void;
		onError?: (error?: Error | string | unknown) => void;
	}
) => {
	if (!file) return;

	const formData = new FormData();
	formData.append('file', file);

	try {
		const xhr = new XMLHttpRequest();
		xhr.open('POST', endpoint);

		xhr.upload.onprogress = (event: ProgressEvent) => {
			if (event.lengthComputable) {
				const progress = Math.round((event.loaded * 100) / event.total);
				callbacks?.onProgress?.(progress);
			}
		};

		xhr.onload = () => {
			if (xhr.status === 200 && xhr.responseText !== 'FAIL') {
				callbacks?.onSuccess?.();
			} else {
				callbacks?.onError?.(xhr.responseText);
			}
		};

		xhr.onerror = () => {
			callbacks?.onError?.('Network error');
		};

		xhr.send(formData);
	} catch (error) {
		callbacks?.onError?.(error);
		console.error(error);
	}
};

export const uploadFirmwareFile = (
	firmwareUploadFile: File,
	callbacks?: {
		onProgress?: (progress: number) => void;
		onSuccess?: () => void;
		onError?: (error?: Error | string | unknown) => void;
	}
) => {
	uploadFile(firmwareUploadFile, `${baseUrl}/upload/firmware`, callbacks);
};

export const uploadWebUiFile = (
	firmwareWebUiFile: File,
	callbacks?: {
		onProgress?: (progress: number) => void;
		onSuccess?: () => void;
		onError?: (error?: Error | string | unknown) => void;
	}
) => {
	uploadFile(firmwareWebUiFile, `${baseUrl}/upload/webui`, callbacks);
};

/**
 * Automatically updates the firmware
 */
export const autoUpdate = async (callbacks?: {
	onSuccess?: (message: string) => void;
	onError?: (error?: Error | string | unknown) => void;
}) => {
	try {
		const response = await fetch(`${baseUrl}/api/firmware/auto_update`);
		const data = await response.json();
		const message = data.msg;

		if (!response.ok) {
			callbacks?.onError?.(message);
		} else {
			callbacks?.onSuccess?.(message);
		}
	} catch (error) {
		callbacks?.onError?.(error);
		console.error('Error during auto-update:', error);
	}
};
