import { writable } from 'svelte/store';
import { settings } from './settings';
import { get } from 'svelte/store';

export interface FirmwareReleaseAsset {
	name: string;
	browser_download_url: string;
}

export interface FirmwareRelease {
	tag_name: string;
	published_at: string;
	html_url: string;
	assets: FirmwareReleaseAsset[];
	isLoading: boolean;
	error: string | null;
}

const defaultFirmwareRelease: FirmwareRelease = {
	tag_name: '',
	published_at: '',
	html_url: '',
	assets: [],
	isLoading: false,
	error: null
};

function createFirmwareStore() {
	const { subscribe, set, update } = writable<FirmwareRelease>(defaultFirmwareRelease);

	return {
		subscribe,
		fetchLatest: async () => {
			update((state) => ({ ...state, isLoading: true, error: null }));

			try {
				const settingsValue = get(settings);
				const url =
					settingsValue.gitReleaseUrl ||
					'https://git.btclock.dev/api/v1/repos/btclock/btclock_v3/releases/latest';

				const response = await fetch(url);

				if (!response.ok) {
					throw new Error(`Error fetching firmware: ${response.statusText}`);
				}

				const data = await response.json();

				// Format the data to match our store structure
				const release: FirmwareRelease = {
					tag_name: data.tag_name,
					published_at: data.published_at,
					html_url: data.html_url,
					assets: data.assets.map((asset: { name: string; browser_download_url: string }) => ({
						name: asset.name,
						browser_download_url: asset.browser_download_url
					})),
					isLoading: false,
					error: null
				};

				set(release);
				return release;
			} catch (error) {
				const errorMessage = error instanceof Error ? error.message : 'Unknown error';
				console.error('Failed to fetch firmware info:', errorMessage);

				update((state) => ({
					...state,
					isLoading: false,
					error: errorMessage
				}));

				return null;
			}
		},
		reset: () => set(defaultFirmwareRelease)
	};
}

export const firmwareRelease = createFirmwareStore();
