import { writable } from 'svelte/store';
import { baseUrl } from '$lib/env';
import type { Status } from '$lib/types';
// Create a default status object
const defaultStatus: Status = {
	currentScreen: 0,
	numScreens: 0,
	timerRunning: false,
	isOTAUpdating: false,
	espUptime: 0,
	espFreeHeap: 0,
	espHeapSize: 0,
	connectionStatus: {
		price: false,
		blocks: false,
		V2: false,
		nostr: false
	},
	rssi: 0,
	currency: 'USD',
	dnd: {
		enabled: false,
		timeBasedEnabled: false,
		startTime: '00:00',
		endTime: '00:00',
		active: false
	},
	data: [],
	leds: [
		{ red: 0, green: 0, blue: 0, hex: '#000000' },
		{ red: 0, green: 0, blue: 0, hex: '#000000' },
		{ red: 0, green: 0, blue: 0, hex: '#000000' },
		{ red: 0, green: 0, blue: 0, hex: '#000000' }
	]
};

// Create the Svelte store
function createStatusStore() {
	const { subscribe, set, update } = writable<Status>(defaultStatus);
	let eventSource: EventSource | null = null;

	// Clean up function to close SSE connection
	const cleanup = () => {
		if (eventSource) {
			eventSource.close();
			eventSource = null;
		}
	};

	// Create the store object with methods
	const store = {
		subscribe,
		fetch: async () => {
			try {
				const response = await fetch(`${baseUrl}/api/status`);
				if (!response.ok) {
					throw new Error(`Error fetching status: ${response.statusText}`);
				}
				const data = await response.json();
				data.isLoaded = true;
				set(data);
				return data;
			} catch (error) {
				console.error('Failed to fetch status:', error);
				return defaultStatus;
			}
		},
		startListening: () => {
			// Clean up any existing connections first
			cleanup();

			// Only run in the browser, not during SSR
			if (typeof window === 'undefined') return;

			try {
				// Create a new EventSource connection
				eventSource = new EventSource(`${baseUrl}/events`);

				// Handle status updates
				eventSource.addEventListener('status', (event) => {
					try {
						const data = JSON.parse(event.data);
						update((currentStatus) => ({ ...currentStatus, ...data }));
					} catch (error) {
						console.error('Error processing status event:', error);
					}
				});

				// Handle connection status updates
				eventSource.addEventListener('connection', (event) => {
					try {
						const data = JSON.parse(event.data);
						update((currentStatus) => ({
							...currentStatus,
							connectionStatus: {
								...currentStatus.connectionStatus,
								...data
							}
						}));
					} catch (error) {
						console.error('Error processing connection event:', error);
					}
				});

				// Handle screen updates
				eventSource.addEventListener('screen', (event) => {
					try {
						const data = JSON.parse(event.data);
						update((currentStatus) => ({
							...currentStatus,
							currentScreen: data.screen || currentStatus.currentScreen
						}));
					} catch (error) {
						console.error('Error processing screen event:', error);
					}
				});

				// Handle generic messages
				eventSource.onmessage = (event) => {
					if (event.type === 'message') {
						return;
					}
					try {
						const data = JSON.parse(event.data);
						update((currentStatus) => ({ ...currentStatus, ...data }));
					} catch (error) {
						console.error('Error processing message event:', error);
					}
				};

				// Handle errors
				eventSource.onerror = (error) => {
					console.error('EventSource failed:', error);
					cleanup();
					// Attempt to reconnect after a delay
					setTimeout(() => store.startListening(), 5000);
				};
			} catch (error) {
				console.error('Failed to setup event source:', error);
			}
		},
		stopListening: cleanup,

		// Function to set the current screen
		setScreen: async (id: number): Promise<boolean> => {
			try {
				// Make the GET request to change the screen
				const response = await fetch(`${baseUrl}/api/show/screen/${id}`);

				if (!response.ok) {
					throw new Error(`Error setting screen: ${response.statusText}`);
				}

				// Update the store with the new screen ID
				update((currentStatus) => ({
					...currentStatus,
					currentScreen: id
				}));

				return true;
			} catch (error) {
				console.error('Failed to set screen:', error);
				return false;
			}
		}
	};

	return store;
}

export const status = createStatusStore();

// Initialize the store by fetching initial status and starting to listen for updates
if (typeof window !== 'undefined') {
	status.fetch().then(() => status.startListening());

	// Clean up the EventSource when the window is unloaded
	window.addEventListener('beforeunload', () => {
		status.stopListening();
	});
}
