export const toTime = (secs: number) => {
	const hours = Math.floor(secs / (60 * 60));

	const divisor_for_minutes = secs % (60 * 60);
	const minutes = Math.floor(divisor_for_minutes / 60);

	const divisor_for_seconds = divisor_for_minutes % 60;
	const seconds = Math.ceil(divisor_for_seconds);

	const obj = {
		h: hours,
		m: minutes,
		s: seconds
	};
	return obj;
};

export const toUptimestring = (secs: number): string => {
	const time = toTime(secs);

	return `${time.h}h ${time.m}m ${time.s}s`;
};

/**
 * Determines the appropriate firmware binary name based on hardware revision
 * @param hwRev Hardware revision string
 * @returns The firmware binary filename appropriate for this hardware
 */
export const getFirmwareBinaryName = (hwRev: string) => {
	let binaryFilename = '';
	switch (hwRev) {
		case 'REV_V8_EPD_2_13':
			binaryFilename = 'btclock_rev_v8_213epd_firmware.bin';
			break;
		case 'REV_B_EPD_2_13':
			binaryFilename = 'btclock_rev_b_213epd_firmware.bin';
			break;
		case 'REV_A_EPD_2_13':
			binaryFilename = 'lolin_s3_mini_213epd_firmware.bin';
			break;
		case 'REV_A_EPD_2_9':
			binaryFilename = 'lolin_s3_mini_29epd_firmware.bin';
			break;
		default:
			binaryFilename = 'Unsupported hardware, unable to determine firmware binary filename';
	}

	return binaryFilename;
};

/**
 * Determines the appropriate WebUI binary name based on hardware revision
 * @param hwRev Hardware revision string
 * @returns The WebUI binary filename appropriate for this hardware
 */
export const getWebUiBinaryName = (hwRev: string) => {
	let webuiFilename = '';
	switch (hwRev) {
		case 'REV_V8_EPD_2_13':
			webuiFilename = 'littlefs_16MB.bin';
			break;
		case 'REV_B_EPD_2_13':
			webuiFilename = 'littlefs_8MB.bin';
			break;
		case 'REV_A_EPD_2_13':
			webuiFilename = 'littlefs_4MB.bin';
			break;
		default:
			webuiFilename = 'Unsupported hardware, unable to determine WebUI binary filename';
	}

	return webuiFilename;
};

export const compareVersions = (version1: string, version2: string): number => {
	if (!version2) return 0;

	const parts1 = version1.split('.').map((part) => parseInt(part, 10));
	const parts2 = version2.split('.').map((part) => parseInt(part, 10));

	for (let i = 0; i < 3; i++) {
		if (parts1[i] > parts2[i]) {
			return 1;
		} else if (parts1[i] < parts2[i]) {
			return -1;
		}
	}

	return 0;
};

export const miningPoolMap = new Map<string, string>([
	['noderunners', 'Noderunners.network'],
	['braiins', 'Braiins Pool'],
	['ocean', 'ocean.xyz'],
	['satoshi_radio', 'Satoshi Radio pool'],
	['public_pool', 'public-pool.io'],
	['gobrrr_pool', 'Go Brrr pool'],
	['ckpool', 'CKPool'],
	['eu_ckpool', 'EU CKPool'],
	['local_public_pool', 'Public Pool (local)']
]);
